@@ -1,7 +1,7 @@
 ;;; cider-eval.el --- Interactive evaluation (compilation) functionality -*- lexical-binding: t -*-
 
 ;; Copyright © 2012-2013 Tim King, Phil Hagelberg, Bozhidar Batsov
-;; Copyright © 2013-2018 Bozhidar Batsov, Artur Malabarba and CIDER contributors
+;; Copyright © 2013-2019 Bozhidar Batsov, Artur Malabarba and CIDER contributors
 ;;
 ;; Author: Tim King <kingtim@gmail.com>
 ;;         Phil Hagelberg <technomancy@gmail.com>
@@ -168,7 +168,8 @@ When invoked with a prefix ARG the command doesn't prompt for confirmation."
 (defun cider--quit-error-window ()
   "Buries the `cider-error-buffer' and quits its containing window."
   (when-let* ((error-win (get-buffer-window cider-error-buffer)))
-    (quit-window nil error-win)))
+    (save-excursion
+      (quit-window nil error-win))))
 
 
 ;;; Dealing with compilation (evaluation) errors and warnings
@@ -377,7 +378,8 @@ evaluation command.  Honor `cider-auto-jump-to-error'."
     (let* ((face (nth 3 info))
            (note (nth 4 info))
            (auto-jump (if (eq cider-auto-jump-to-error 'errors-only)
-                          (not (eq face 'cider-warning-highlight-face))
+                          (not (or (eq face 'cider-warning-highlight-face)
+                                   (string-match-p "warning" note)))
                         cider-auto-jump-to-error)))
       (overlay-put overlay 'cider-note-p t)
       (overlay-put overlay 'font-lock-face face)
