@@ -68,7 +68,7 @@
     (it "uses cider-reader-conditional-face"
       (spy-on 'cider-connected-p :and-return-value t)
       (spy-on 'cider-repls :and-return-value '(list t))
-      (spy-on 'cider-repl-type :and-return-value "clj")
+      (spy-on 'cider-repl-type :and-return-value 'clj)
       (cider--test-with-temp-buffer "#?(:clj 'clj :cljs 'cljs :cljr 'cljr)"
         (let ((cider-font-lock-reader-conditionals t)
               (found (cider--face-exists-in-range-p (point-min) (point-max)
@@ -78,7 +78,7 @@
     (it "highlights unmatched reader conditionals"
       (spy-on 'cider-connected-p :and-return-value t)
       (spy-on 'cider-repls :and-return-value '(list t))
-      (spy-on 'cider-repl-type :and-return-value "clj")
+      (spy-on 'cider-repl-type :and-return-value 'clj)
       (cider--test-with-temp-buffer "#?(:clj 'clj :cljs 'cljs :cljr 'cljr)"
         (let ((cider-font-lock-reader-conditionals t))
           (expect (cider--face-exists-in-range-p 4 12 'cider-reader-conditional-face)
@@ -91,7 +91,7 @@
     (it "works with splicing"
       (spy-on 'cider-connected-p :and-return-value t)
       (spy-on 'cider-repls :and-return-value '(list t))
-      (spy-on 'cider-repl-type :and-return-value "clj")
+      (spy-on 'cider-repl-type :and-return-value 'clj)
       (cider--test-with-temp-buffer "[1 2 #?(:clj [3 4] :cljs [5 6] :cljr [7 8])]"
         (let ((cider-font-lock-reader-conditionals t))
           (expect (cider--face-exists-in-range-p 1 18 'cider-reader-conditional-face)
@@ -104,7 +104,7 @@
     (it "does not apply inside strings or comments"
       (spy-on 'cider-connected-p :and-return-value t)
       (spy-on 'cider-repls :and-return-value '(list t))
-      (spy-on 'cider-repl-type :and-return-value "clj")
+      (spy-on 'cider-repl-type :and-return-value 'clj)
       (cider--test-with-temp-buffer "\"#?(:clj 'clj :cljs 'cljs :cljr 'cljr)\" ;; #?(:clj 'clj :cljs 'cljs :cljr 'cljr)"
         (let ((cider-font-lock-reader-conditionals t))
           (expect (cider--face-exists-in-range-p (point-min) (point-max) 'cider-reader-conditional-face)
@@ -113,7 +113,7 @@
     (it "highlights all unmatched reader conditionals"
       (spy-on 'cider-connected-p :and-return-value t)
       (spy-on 'cider-repls :and-return-value '(list t))
-      (spy-on 'cider-repl-type :and-return-value "cljs")
+      (spy-on 'cider-repl-type :and-return-value 'cljs)
       (cider--test-with-temp-buffer
           "#?(:clj 'clj :cljs 'cljs :cljr 'cljr)\n#?(:clj 'clj :cljs 'cljs :cljr 'cljr)\n"
         (let ((cider-font-lock-reader-conditionals t))
@@ -129,7 +129,7 @@
     (it "does not highlight beyond the limits of the reader conditional group"
       (spy-on 'cider-connected-p :and-return-value t)
       (spy-on 'cider-repls :and-return-value '(list t))
-      (spy-on 'cider-repl-type :and-return-value "clj")
+      (spy-on 'cider-repl-type :and-return-value 'clj)
       (cider--test-with-temp-buffer
           "#?(:clj 'clj :cljs 'cljs :cljr 'cljr)\n#?(:clj 'clj :cljs 'cljs :cljr 'cljr)\n"
         (let ((cider-font-lock-reader-conditionals t))
@@ -144,7 +144,7 @@
     (it "is disabled"
       (spy-on 'cider-connected-p :and-return-value nil)
       (spy-on 'cider-repls :and-return-value '(list t))
-      (spy-on 'cider-repl-type :and-return-value '("clj" "cljs"))
+      (spy-on 'cider-repl-type :and-return-value '(clj cljs))
       (cider--test-with-temp-buffer "#?(:clj 'clj :cljs 'cljs :cljr 'cljr)"
         (let ((cider-font-lock-reader-conditionals t))
           (expect (cider--face-exists-in-range-p (point-min) (point-max) 'cider-reader-conditional-face)
